/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2015 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "fixedValueProfileFvPatchField.H"

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

template<class Type>
Foam::fixedValueProfileFvPatchField<Type>::fixedValueProfileFvPatchField
(
    const fvPatch& p,
    const DimensionedField<Type, volMesh>& iF
)
:
    fixedValueFvPatchField<Type>(p, iF),
    fixedValueProfile_(),
    dir_(pTraits<vector>::zero),
    refPoint_(pTraits<vector>::zero)
{}


template<class Type>
Foam::fixedValueProfileFvPatchField<Type>::fixedValueProfileFvPatchField
(
    const fvPatch& p,
    const DimensionedField<Type, volMesh>& iF,
    const Field<Type>& fld
)
:
    fixedValueFvPatchField<Type>(p, iF, fld),
    fixedValueProfile_(),
    dir_(pTraits<vector>::zero),
    refPoint_(pTraits<vector>::zero)

{}


template<class Type>
Foam::fixedValueProfileFvPatchField<Type>::fixedValueProfileFvPatchField
(
    const fixedValueProfileFvPatchField<Type>& ptf,
    const fvPatch& p,
    const DimensionedField<Type, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    fixedValueFvPatchField<Type>(p, iF),  // bypass mapper
    fixedValueProfile_(ptf.fixedValueProfile_().clone().ptr()),
    dir_(ptf.dir_),
    refPoint_(ptf.refPoint_)

{
    // Evaluate since value not mapped
    dir_ /= mag(dir_);
    const scalarField dirCmpt = dir_ & (this->patch().Cf() - refPoint_);
    fvPatchField<Type>::operator==(fixedValueProfile_->value(dirCmpt));
}


template<class Type>
Foam::fixedValueProfileFvPatchField<Type>::fixedValueProfileFvPatchField
(
    const fvPatch& p,
    const DimensionedField<Type, volMesh>& iF,
    const dictionary& dict
)
:
    fixedValueFvPatchField<Type>(p, iF),
    fixedValueProfile_(DataEntry<Type>::New("valueProfile", dict)),
    dir_(dict.lookup("direction")),
    refPoint_(dict.lookup("refPoint"))

{
    if (mag(dir_) < SMALL)
    {
        FatalErrorIn
        (
            "fixedValueProfileFvPatchField(const dictionary&)"
        )   << "magnitude Direction must be greater than zero"
            << abort(FatalError);
    }

    // Ensure direction vectors are normalized
    dir_ /= mag(dir_);

    const scalarField dirCmpt = dir_ & (this->patch().Cf() - refPoint_);
    fvPatchField<Type>::operator==(fixedValueProfile_->value(dirCmpt));
}


template<class Type>
Foam::fixedValueProfileFvPatchField<Type>::fixedValueProfileFvPatchField
(
    const fixedValueProfileFvPatchField<Type>& ptf
)
:
    fixedValueFvPatchField<Type>(ptf),
    fixedValueProfile_
    (
        ptf.fixedValueProfile_.valid()
      ? ptf.fixedValueProfile_().clone().ptr()
      : NULL
    ),
    dir_(ptf.dir_),
    refPoint_(ptf.refPoint_)
{}


template<class Type>
Foam::fixedValueProfileFvPatchField<Type>::fixedValueProfileFvPatchField
(
    const fixedValueProfileFvPatchField<Type>& ptf,
    const DimensionedField<Type, volMesh>& iF
)
:
    fixedValueFvPatchField<Type>(ptf, iF),
    fixedValueProfile_
    (
        ptf.fixedValueProfile_.valid()
      ? ptf.fixedValueProfile_().clone().ptr()
      : NULL
    ),
    dir_(ptf.dir_),
    refPoint_(ptf.refPoint_)
{
    // For safety re-evaluate
    const scalarField dirCmpt = dir_ & (this->patch().Cf() - refPoint_);

    if (ptf.fixedValueProfile_.valid())
    {
        fvPatchField<Type>::operator==(fixedValueProfile_->value(dirCmpt));
    }
}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

template<class Type>
void Foam::fixedValueProfileFvPatchField<Type>::updateCoeffs()
{
    if (this->updated())
    {
        return;
    }

    const scalarField dirCmpt = dir_ & (this->patch().Cf() - refPoint_);
    fvPatchField<Type>::operator==(fixedValueProfile_->value(dirCmpt));

    fixedValueFvPatchField<Type>::updateCoeffs();
}


template<class Type>
void Foam::fixedValueProfileFvPatchField<Type>::write(Ostream& os) const
{
    fvPatchField<Type>::write(os);
    fixedValueProfile_->writeData(os);
    os.writeKeyword("direction")
    << dir_ << token::END_STATEMENT << nl;
    os.writeKeyword("refPoint")
    << refPoint_ << token::END_STATEMENT << nl;
    this->writeEntry("value", os);
}


// ************************************************************************* //
